@@ -1,6 +1,6 @@
   /*
 
-  knetfilter v. 3.1.0
+  knetfilter v. 3.1.2
 
   Copyright (C) 2002 Luigi Genoni
   		     venom@sns.it
@@ -102,64 +102,137 @@ void knetfilter::startmangleRules() {
 
     if (strcmp(mangleactionChoice,"TOS")==0)
       { 
-	if (strcmp(mangleprotocolChoice,"icmp")==0)
+	if (strcmp(mangleprotocolChoice,"tcp")==0)
 	  {
-	    if(firstmangleport.length()==0) // fp=n
-	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
-	      }
+	    if (strcmp(hasmangleEcn,"ecn")==0)
+		{
+		  if(firstmangleport.length()==0) // fp=n
+		    {	
+		      if(secondmangleport.length()==0) // fp=n sp=n
+			{
+			  addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+			}
+		      else // fp=n sp!=n
+			{ 
+			  addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+			}
+		    }
+		  else //fp!=n
+		    {	    
+		      if(secondmangleport.length()==0) // fp!=n sp=n
+			{
+			  addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+			}
+		      else // fp!=n sp!=n
+			{
+			  addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+			}	
+		    }	
+		}
 	    else
 	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
-	      }
-	  }	
-	else
-	  {
-	    if(firstmangleport.length()==0) // fp=n
-	      {	
-		if(secondmangleport.length()==0) // fp=n sp=n
-		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		  }
+		else //fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
 		      }
-	    else // fp=n sp!=n
-	      { 
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
 		  }
 	      }
-	    else //fp!=n
-	      {	    
-	    if(secondmangleport.length()==0) // fp!=n sp=n
+	  }
+	else
+	  { 
+	    if (strcmp(mangleprotocolChoice,"icmp")==0)
 	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		if(firstmangleport.length()==0) // fp=n
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		  }
+		else
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
 		  }
-	    else // fp!=n sp!=n
-	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
-		  }	
 	      }	
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		  }
+		else //fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "TOS" << "--set-tos" << mangletosChoice;
+		      }	
+		  }	
+	      }
 	  }
       };
 
     if (strcmp(mangleactionChoice,"DSCP")==0)
       { 
-	if(mangledscp.length()==0)
+	if(strcmp(mangleprotocolChoice,"tcp")==0)
 	  {
-	    QMessageBox::critical(0, "Error",
-				  "You have to supply a DSCP esadecimal value to set.\n");
-	  }
-	else
-	  {
-	    if (strcmp(mangleprotocolChoice,"icmp")==0)
+	    if (strcmp(hasmangleEcn,"ecn")==0)
 	      {
-		if(firstmangleport.length()==0) // fp=n
+		if(mangledscp.length()==0)
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		    QMessageBox::critical(0, "Error",
+					  "You have to supply a DSCP esadecimal value to set.\n");
 		  }
 		else
-		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		  {		    
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+		      }
+		    else //fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }	
+		      }	
 		  }
-	      }	
+	      }
 	    else
 	      {
 		if(firstmangleport.length()==0) // fp=n
@@ -182,30 +255,94 @@ void knetfilter::startmangleRules() {
 		    else // fp!=n sp!=n
 		      {
 			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
-		      }	
-		  }	
+		      }
+		  }	    
 	      }
 	  }
-      };
-
-    if (strcmp(mangleactionChoice,"MARK")==0)
-      {
-	if (manglemark.length()==0)
-	  {
-	    QMessageBox::critical(0, "Error",
-				  "You have to supply a MARK to set.\n");
-	  }
 	else
 	  {
-	    if (strcmp(mangleprotocolChoice,"icmp")==0)
+	    if(mangledscp.length()==0)
 	      {
-		if(firstmangleport.length()==0)
+		QMessageBox::critical(0, "Error",
+				      "You have to supply a DSCP esadecimal value to set.\n");
+	      }
+	    else
+	      {
+		if (strcmp(mangleprotocolChoice,"icmp")==0)
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+		    if(firstmangleport.length()==0) // fp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		    else
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		  }	
+		else
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+		      }
+		    else //fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }	
+		      }	
+		  }
+	      }
+	  }
+      };
+    
+    if (strcmp(mangleactionChoice,"MARK")==0)
+      {
+	if (strcmp(mangleprotocolChoice,"tcp")==0)
+	  {
+	    if (strcmp(hasmangleEcn,"ecn")==0)
+	      {
+		if (manglemark.length()==0)
+		  {
+		    QMessageBox::critical(0, "Error",
+					  "You have to supply a MARK to set.\n");
 		  }
 		else
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+		      }
+		    else // fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }	
+		      }	
 		  }
 	      }
 	    else
@@ -231,71 +368,191 @@ void knetfilter::startmangleRules() {
 		      {
 			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "MARK" << "--set-mark"  << manglemark; 
 		      }	
-		  }	
-	      }
-	  }
-      };
-
-    if (strcmp(mangleactionChoice,"LOG")==0) 
-      {
-	if (strcmp(mangleprotocolChoice,"icmp")==0)
-	  {
-	    if(firstmangleport.length()==0) // fp=n
-	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
-	      }
-	    else // fp!=n
-	      {
-		addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		  }
 	      }
 	  }
 	else
 	  {
-	    if(firstmangleport.length()==0) // fp=n
-	      {	
-		if(secondmangleport.length()==0) // fp=n sp=n
+	    if (manglemark.length()==0)
+	      {
+		QMessageBox::critical(0, "Error",
+				      "You have to supply a MARK to set.\n");
+	      }
+	    else
+	      {
+		if (strcmp(mangleprotocolChoice,"icmp")==0)
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		    if(firstmangleport.length()==0)
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+		      }
+		    else
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+		      }
 		  }
-		else // fp=n sp!=n
-		  { 
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		else
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+		      }
+		    else // fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "MARK" << "--set-mark"  << manglemark; 
+			  }	
+		      }	
 		  }
 	      }
-	    else // fp!=n
-	      {	    
-		if(secondmangleport.length()==0) // fp!=n sp=n
-		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
-		  }
-		else // fp!=n sp!=n
-		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
-		  }	
-	      }	
-	  }	
-      };
-
-    if (strcmp(mangleactionChoice,"ULOG")==0) 
-      {
-	if (strcmp(mangleulogNlgroup,"0")==0) 
-	  {
-	    QMessageBox::critical(0, "Error",
-				  "NetLink group should be beetwen 1 and 32\n");
 	  }
-	else	  
+      };
+    
+    if (strcmp(mangleactionChoice,"LOG")==0) 
+      {
+	if(strcmp(mangleprotocolChoice,"tcp")==0)
+	  {
+	    if (strcmp(hasmangleEcn,"ecn")==0)
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		  }
+		else // fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }	
+		  }	
+	      }
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		  }
+		else // fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }	
+		  }	
+	      }
+	  }
+	else
 	  {
 	    if (strcmp(mangleprotocolChoice,"icmp")==0)
 	      {
 		if(firstmangleport.length()==0) // fp=n
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
 		  }
 		else // fp!=n
 		  {
-		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
 		  }
 	      }
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		  }
+		else // fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "LOG" << "--log-level" << logLevel << "--log-prefix" << manglelogname << log1 << log2 << log3;  
+		      }	
+		  }	
+	      }
+	  }
+      };
+
+    if (strcmp(mangleactionChoice,"ULOG")==0) 
+      {
+	if (strcmp(mangleprotocolChoice,"tcp")==0)
+	  {
+	    if (strcmp(hasmangleEcn,"ecn")==0)
+	      {
+		if (strcmp(mangleulogNlgroup,"0")==0) 
+		  {
+		    QMessageBox::critical(0, "Error",
+					  "NetLink group should be beetwen 1 and 32\n");
+		  }
+		else	  
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-prefix" << mangleulogname;  
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--log-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }
+		      }
+		    else // fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-m" << hasmangleEcn << mangleecnflagChoice << mangleecnParam << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }	
+		      }	
+		  }	
+	      }
 	    else
 	      {
 		if(firstmangleport.length()==0) // fp=n
@@ -320,7 +577,54 @@ void knetfilter::startmangleRules() {
 			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
 		      }	
 		  }	
-	      }	
+	      }
+	  } 
+	else
+	  {
+	    if (strcmp(mangleulogNlgroup,"0")==0) 
+	      {
+		QMessageBox::critical(0, "Error",
+				      "NetLink group should be beetwen 1 and 32\n");
+	      }
+	    else	  
+	      {
+		if (strcmp(mangleprotocolChoice,"icmp")==0)
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		      }
+		    else // fp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		      }
+		  }
+		else
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-prefix" << mangleulogname;  
+			  }
+			else // fp=n sp!=n
+			  { 
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--log-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }
+		      }
+		    else // fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }
+			else // fp!=n sp!=n
+			  {
+			    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+			  }	
+		      }	
+		  }	
+	      }
 	  }
       };
 
@@ -813,10 +1117,31 @@ void knetfilter::mangleRules() {
       connect(mangleaction,SIGNAL(activated(int)), SLOT(mangleactionActivated(int))); 
       grid->addMultiCellWidget(mangleaction,4,4,2,3);
 
+      QLabel *labelmangleEcn = new QLabel(mangleList);
+      labelmangleEcn->setText("ECN TCP/IP match flags:");
+      grid->addMultiCellWidget(labelmangleEcn,4,4,4,5);
+
+      mangleecnflags = new QComboBox( FALSE, mangleList, "ECNFlags");
+      mangleecnflags->insertItem(" ");
+      mangleecnflags->insertItem("CWR bit of TCP header");
+      mangleecnflags->insertItem("ECE bit of TCP header");
+      mangleecnflags->insertItem("ECN codepoint 0 in IPv4 header");
+      mangleecnflags->insertItem("ECN codepoint 1 in IPv4 header");
+      mangleecnflags->insertItem("ECN codepoint 2 in IPv4 header");
+      mangleecnflags->insertItem("ECN codepoint 3 in IPv4 header");
+      mangleecnflags->setFixedHeight(mangleecnflags->sizeHint().height());
+      connect(mangleecnflags,SIGNAL(activated(int)), SLOT(mangleecnflagsActivated(int)));
+
+      grid->addMultiCellWidget(mangleecnflags,4,4,6,7);
+
       QLabel *labeladvise = new QLabel(mangleList);
       labeladvise->setText("Set the choose Flag related options, others will be ignored ");
       grid->addMultiCellWidget(labeladvise,5,5,1,9);      
 
+      QLabel *labelmangleMark = new QLabel(mangleList);
+      labelmangleMark->setText("MARK:");
+      grid->addMultiCellWidget(labelmangleMark,6,6,5,5);
+
       QLabel *labelmangleTos = new QLabel(mangleList);
       labelmangleTos->setText("TOS:");
       grid->addMultiCellWidget(labelmangleTos,6,6,0,1);
@@ -831,10 +1156,6 @@ void knetfilter::mangleRules() {
       connect(mangletos,SIGNAL(activated(int)), SLOT(mangletosActivated(int)));
       grid->addMultiCellWidget(mangletos,6,6,1,3);
 
-      QLabel *labelmangleMark = new QLabel(mangleList);
-      labelmangleMark->setText("MARK:");
-      grid->addMultiCellWidget(labelmangleMark,6,6,5,5);
-
       mangleMark = new QLineEdit(mangleList);
 
       mangleMark->setFixedHeight(mangleMark->sizeHint().height());
@@ -1341,6 +1662,51 @@ void knetfilter::manglepolicyActivated(int index) {
   };
 };
 
+void knetfilter::mangleecnflagsActivated(int index) {
+  switch(index) {
+  default:
+    hasmangleEcn = "-v";
+    mangleecnflagChoice = "-v";
+    mangleecnParam = "-v";
+  case 0:
+    hasmangleEcn = "-v";
+    mangleecnflagChoice = "-v";
+    mangleecnParam = "-v";
+    break;
+  case 1:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-tcp-cwr";
+    mangleecnParam = "-v";
+    break;
+  case 2:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-tcp-ece";
+    mangleecnParam = "-v";
+    break;
+  case 3:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-ip-ect";
+    mangleecnParam = "0";
+    break;
+  case 4:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-ip-ect";
+    mangleecnParam = "1";
+      break;
+  case 5:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-ip-ect";
+    mangleecnParam = "2";
+      break;
+  case 6:
+    hasmangleEcn = "ecn";
+    mangleecnflagChoice = "--ecn-ip-ect";
+    mangleecnParam = "3";
+      break;
+  };
+};
+
+
 void knetfilter::closemanglepolRules() {
 
   manglepolList->close();
